#header {
  margin-left:5px;
  width:50%;
  overflow: hidden;
}

#title h1 {
  display:inline;
}

#admin {
  float: right;
}

#rhs {
    margin-top:0;
    margin-left:10px;
    margin-right:10px;
}

#confirm-collapse {
    margin-top:3px;
}

#action-pane {
    margin-top:15px;
}

#custom-select-pane {
    margin-top:15px;
}
