{% load staticfiles %}
<!DOCTYPE html>
<html>
<head>
  <script type='text/javascript' 
    src='http://code.jquery.com/jquery-1.11.3.min.js'></script>
  <link rel="stylesheet"
    href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.5/css/bootstrap.min.css"
    />
  <script
    src="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.5/js/bootstrap.min.js">
  </script>

  <script src="{% static 'flowr/js/cytoscape.min.js' %}"></script>

  <link rel="stylesheet" type="text/css" 
    href="{% static 'flowr/css/flowr.css' %}"/>

  {% block extra_head %}
  {% endblock extra_head %}
</head>
<body>
  {% block body %}
  {% endblock body %}

  {% block body_post %}
  {% endblock body_post %}
</body>
</html>
