{% extends "flowr/base.html" %}

{% block body %}
  {% include "flowr/header.html" %}
  <div id="cy" style="height:500px; width:500px">
  </div>
{% endblock body %}

{% block body_post %}
<script type="text/javascript">
  {% include "flowr/graph.js" %}
</script>
{% endblock body_post %}
